@@ -17,7 +17,7 @@ namespace tf {
 /**
 @class cudaFlowCapturer
 
-@brief class for building a CUDA task dependency graph through stream capture
+@brief class to create a %cudaFlow graph using stream capture
 
 The usage of tf::cudaFlowCapturer is similar to tf::cudaFlow, except users can
 call the method tf::cudaFlowCapturer::on to capture a sequence of asynchronous 
@@ -293,10 +293,10 @@ class cudaFlowCapturer {
 
     @tparam C callable type
 
-    @param callable callable to run by a single kernel thread
+    @param c callable to run by a single kernel thread
     */
     template <typename C>
-    cudaTask single_task(C callable);
+    cudaTask single_task(C c);
     
     /**
     @brief updates a capture task to a single-threaded kernel
@@ -305,7 +305,7 @@ class cudaFlowCapturer {
     on an existing task.
     */
     template <typename C>
-    void single_task(cudaTask task, C callable);
+    void single_task(cudaTask task, C c);
     
     /**
     @brief captures a kernel that applies a callable to each dereferenced element 
@@ -1057,7 +1057,7 @@ inline cudaFlowCapturer::cudaFlowCapturer(cudaGraph& g) :
 // constructs a standalone cudaFlow capturer
 inline cudaFlowCapturer::cudaFlowCapturer() : 
   _handle {std::in_place_type_t<External>{}},
-  _graph  {std::get<External>(_handle).graph} {
+  _graph  {std::get_if<External>(&_handle)->graph} {
 }
 
 inline cudaFlowCapturer::~cudaFlowCapturer() {
@@ -1209,9 +1209,9 @@ void cudaFlowCapturer::offload_until(P&& predicate) {
         cudaGraphInstantiate(&_executable, g, nullptr, nullptr, 0),
         "failed to re-create an executable graph after updates fail"
       );
-      // TODO: store the native graph?
-      TF_CHECK_CUDA(cudaGraphDestroy(g), "failed to destroy captured graph");
     }
+    // TODO: store the native graph?
+    TF_CHECK_CUDA(cudaGraphDestroy(g), "failed to destroy captured graph");
   }
    
   // offload the executable
@@ -1253,7 +1253,8 @@ void cudaFlowCapturer::on(cudaTask task, C&& callable) {
   
   _graph._state |= cudaGraph::UPDATED;
 
-  std::get<cudaNode::Capture>((task._node)->_handle).work = std::forward<C>(callable);
+  std::get_if<cudaNode::Capture>(&task._node->_handle)->work = 
+    std::forward<C>(callable);
 }
 
 // Function: memcpy
