@@ -2,6 +2,7 @@
 
 #include "cuda_memory.hpp"
 #include "cuda_stream.hpp"
+#include "cuda_meta.hpp"
 
 #include "../utility/traits.hpp"
 
@@ -684,7 +685,7 @@ inline void cudaGraph::dump(
 
         case cudaNode::SUBFLOW:
           stack.push(std::make_tuple(
-            &std::get<cudaNode::Subflow>(v->_handle).graph, v, l+1)
+            &(std::get_if<cudaNode::Subflow>(&v->_handle)->graph), v, l+1)
           );
           os << " style=\"filled\""
              << " color=\"black\" fillcolor=\"purple\""
